// 弹幕库实例
var Danmaku = /** @class */ (function () {
    function Danmaku(canvas, danmakuConfig) {
        this.isPause = false;
        this.lastRenderTime = 0;
        this.mDanmuTracks = new Array();
        this.mCanvas = canvas;
        this.mDanmukuConfig = danmakuConfig;
        this.mCtx = canvas.getContext('2d');
        this.initRender();
    }
    Danmaku.prototype.addDanmuTrack = function (danmuTrack) {
        this.mDanmuTracks.push(danmuTrack);
        danmuTrack.setDanmaku(this);
    };
    Danmaku.prototype.removeDanmuTrack = function (danmuTrack) {
        this.mDanmuTracks = this.mDanmuTracks.filter(function (item) { return item !== danmuTrack; });
    };
    Danmaku.prototype.initRender = function () {
        this.mCanvas.width = this.mDanmukuConfig.width;
        this.mCanvas.height = this.mDanmukuConfig.height;
        if (window.requestAnimationFrame) {
            this.renderByAnimationFrame();
        }
        else {
            this.renderBySetInterval();
        }
    };
    Danmaku.prototype.renderByAnimationFrame = function () {
        this.render();
        requestAnimationFrame(this.renderByAnimationFrame.bind(this));
    };
    Danmaku.prototype.renderBySetInterval = function () {
        setInterval(this.render.bind(this), 0);
    };
    Danmaku.prototype.render = function () {
        if (this.isPause)
            return;
        // let currentTime = new Date().getTime()
        // if (currentTime - this.lastRenderTime < 8) {
        //  return
        // }
        // this.lastRenderTime = currentTime
        this.mCtx.clearRect(0, 0, this.mCanvas.width, this.mCanvas.height);
        for (var _i = 0, _a = this.mDanmuTracks; _i < _a.length; _i++) {
            var mDanmuTrack = _a[_i];
            mDanmuTrack.render(this.mCtx);
        }
    };
    Danmaku.prototype.start = function () {
        this.isPause = false;
    };
    Danmaku.prototype.pause = function () {
        this.isPause = true;
    };
    return Danmaku;
}());
//# sourceMappingURL=Danmaku.js.map

var DanmakuPool = /** @class */ (function () {
    function DanmakuPool(size) {
        this.mPoolSize = 20;
        this.mMessagePool = new Array(this.mPoolSize);
        if (size) {
            this.mPoolSize = size;
            this.mMessagePool = new Array(this.mPoolSize);
        }
    }
    DanmakuPool.prototype.addMessage = function (msg) {
        this.addMessages([msg]);
    };
    DanmakuPool.prototype.getMessage = function () {
        var result = null;
        for (var i = 0; i < this.mMessagePool.length; i++) {
            if (this.mMessagePool[i] && this.mMessagePool[i].created) {
                result = this.mMessagePool[i];
                this.mMessagePool[i] = null;
                break;
            }
        }
        return result;
    };
    DanmakuPool.prototype.addMessages = function (msgs) {
        if (msgs === void 0) { msgs = []; }
        for (var i = 0; i < msgs.length; i++) {
            for (var j = 0; j < this.mMessagePool.length; j++) {
                if (!this.mMessagePool[j]) {
                    this.mMessagePool[j] = msgs[i];
                    if (this.mMessagePool[j])
                        this.mMessagePool[j].onBaseCreate();
                    break;
                }
            }
        }
    };
    DanmakuPool.prototype.removeMessage = function (msg) {
        var index = this.mMessagePool.findIndex(function (item) { return item === msg; });
        this.removeMessageByIndex(index);
    };
    DanmakuPool.prototype.removeMessageByIndex = function (index) {
        if (Number.isInteger(index) && index > 0) {
            this.mMessagePool[index] = null;
        }
    };
    return DanmakuPool;
}());
//# sourceMappingURL=DanmakuPool.js.map

var DanmaTrack = /** @class */ (function () {
    function DanmaTrack(danmuPool, danmaTrackConfig) {
        this.mDanmaMessages = new Array();
        this.mTrackInfo = {
            maxWidth: 0,
            top: 0,
            height: 0
        };
        this.mDanmuTrackConfig = {
            top: 0,
            height: 0
        };
        this.mDanmuPool = danmuPool;
        this.initConfig(danmaTrackConfig);
    }
    DanmaTrack.prototype.setDanmaku = function (danmaku) {
        this.mDanmaku = danmaku;
    };
    DanmaTrack.prototype.initConfig = function (danmaTrackConfig) {
        this.mDanmuTrackConfig = danmaTrackConfig;
        this.mTrackInfo.top = danmaTrackConfig.top;
        this.mTrackInfo.height = danmaTrackConfig.height;
    };
    DanmaTrack.prototype.render = function (ctx) {
        this.refreshMessage(ctx);
        for (var i = 0; i < this.mDanmaMessages.length; i++) {
            var currentDanmaMessage = this.mDanmaMessages[i];
            currentDanmaMessage.onBaseMeasure(ctx, this.mTrackInfo);
            currentDanmaMessage.onBaseLayout(ctx, this.mTrackInfo);
            currentDanmaMessage.onBaseDraw(ctx, this.mTrackInfo);
            currentDanmaMessage.onBaseDestroyed();
            if (i === this.mDanmaMessages.length - 1) {
                this.mTrackInfo.maxWidth =
                    currentDanmaMessage.position.left + currentDanmaMessage.size.width;
            }
        }
    };
    DanmaTrack.prototype.refreshMessage = function (ctx) {
        this.removeMessage();
        this.AddMessage(ctx);
    };
    DanmaTrack.prototype.AddMessage = function (ctx) {
        if (this.mTrackInfo.maxWidth < ctx.canvas.width - 10) {
            var msg = this.mDanmuPool.getMessage();
            if (msg) {
                console.log(msg);
                this.mDanmaMessages.push(msg);
            }
        }
    };
    DanmaTrack.prototype.removeMessage = function () {
        this.mDanmaMessages = this.mDanmaMessages.filter(function (item) { return !item.distoryed; });
    };
    return DanmaTrack;
}());
//# sourceMappingURL=DanmaTrack.js.map

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var BaseMessage = /** @class */ (function () {
    function BaseMessage() {
        this.created = false;
        this.distoryed = false;
        this.isLayout = false;
        this.position = {
            top: 0,
            left: 0
        };
        this.size = {
            width: 0,
            height: 0
        };
    }
    BaseMessage.prototype.onBaseCreate = function () {
        var _this = this;
        this.onCreate(function (val) {
            if (val) {
                _this.created = true;
            }
            else {
                _this.distoryed = true;
            }
        });
    };
    BaseMessage.prototype.onBaseDestroyed = function () {
        this.distoryed = this.onDestroyed();
    };
    BaseMessage.prototype.onBaseMeasure = function (ctx, trackInfo) {
        this.size = this.onMeasure(ctx, trackInfo);
    };
    BaseMessage.prototype.onBaseLayout = function (ctx, trackInfo) {
        this.position = this.onLayout(ctx, trackInfo);
    };
    BaseMessage.prototype.onBaseDraw = function (ctx, trackInfo) {
        this.onDraw(ctx, trackInfo);
    };
    return BaseMessage;
}());
//# sourceMappingURL=BaseMessage.js.map

var TextMessage = /** @class */ (function (_super) {
    __extends(TextMessage, _super);
    function TextMessage(msg) {
        var _this = _super.call(this) || this;
        _this.textMessageConfig = {
            mMsg: '',
            color: '#FFF',
            strokeColor: '#000',
            fontSize: 30,
            fontFamily: '黑体'
        };
        _this.textMessageConfig = Object.assign({}, _this.textMessageConfig, msg);
        return _this;
    }
    TextMessage.prototype.onCreate = function (callback) {
        callback(true);
    };
    TextMessage.prototype.onMeasure = function (ctx, trackInfo) {
        ctx.textAlign = 'left';
        ctx.textBaseline = 'middle';
        ctx.font = this.textMessageConfig.fontSize + "px " + this.textMessageConfig.fontFamily;
        var res = ctx.measureText(this.textMessageConfig.mMsg);
        return {
            width: res.width,
            height: trackInfo.height
        };
    };
    TextMessage.prototype.onLayout = function (ctx, trackInfo) {
        if (!this.isLayout) {
            this.isLayout = true;
            return {
                top: trackInfo.top + trackInfo.height / 2,
                left: ctx.canvas.width
            };
        }
        this.position.left -= 3;
        return this.position;
    };
    TextMessage.prototype.onDraw = function (ctx, trackInfo) {
        ctx.fillStyle = this.textMessageConfig.strokeColor;
        ctx.strokeText(this.textMessageConfig.mMsg, this.position.left, this.position.top);
        ctx.fillStyle = this.textMessageConfig.color;
        ctx.fillText(this.textMessageConfig.mMsg, this.position.left, this.position.top);
    };
    TextMessage.prototype.onDestroyed = function () {
        return this.position.left < -this.size.width;
    };
    return TextMessage;
}(BaseMessage));
//# sourceMappingURL=TextMessage.js.map

var ImgMessage = /** @class */ (function (_super) {
    __extends(ImgMessage, _super);
    function ImgMessage(msg) {
        var _this = _super.call(this) || this;
        _this.ImgMessageConfig = {
            url: '',
            width: 0,
            height: 0
        };
        _this.mImg = new Image();
        _this.ImgMessageConfig = Object.assign({}, _this.ImgMessageConfig, msg);
        return _this;
    }
    ImgMessage.prototype.onCreate = function (callback) {
        var _this = this;
        this.mImg.onload = function () {
            callback(true);
        };
        this.mImg.onerror = function () {
            document.body.removeChild(_this.mImg);
            callback(false);
        };
        this.mImg.src = this.ImgMessageConfig.url;
        this.mImg.style.position = 'fixed';
        this.mImg.style.top = '-1000px';
        this.mImg.style.left = '-1000px';
        document.body.append(this.mImg);
    };
    ImgMessage.prototype.onMeasure = function (ctx, trackInfo) {
        return {
            width: this.mImg.width,
            height: this.mImg.height
        };
    };
    ImgMessage.prototype.onLayout = function (ctx, trackInfo) {
        if (!this.isLayout) {
            this.isLayout = true;
            return {
                top: trackInfo.top,
                left: ctx.canvas.width
            };
        }
        this.position.left -= 3;
        return this.position;
    };
    ImgMessage.prototype.onDraw = function (ctx, trackInfo) {
        ctx.drawImage(this.mImg, this.position.left, this.position.top, this.size.width, this.size.height);
    };
    ImgMessage.prototype.onDestroyed = function () {
        var result = this.position.left < -this.size.width;
        if (result) {
            document.body.removeChild(this.mImg);
        }
        return result;
    };
    return ImgMessage;
}(BaseMessage));
//# sourceMappingURL=ImgMessage.js.map

var VideoMessage = /** @class */ (function (_super) {
    __extends(VideoMessage, _super);
    function VideoMessage(msg) {
        var _this = _super.call(this) || this;
        _this.VideoMessageConfig = {
            url: '',
            width: 300,
            height: 200
        };
        _this.mIsPlay = false;
        _this.mVideo = document.createElement('video');
        _this.VideoMessageConfig = Object.assign({}, _this.VideoMessageConfig, msg);
        return _this;
    }
    VideoMessage.prototype.onCreate = function (callback) {
        var _this = this;
        this.mVideo.addEventListener('canplay', function () {
            callback(true);
        });
        this.mVideo.addEventListener('error', function () {
            document.body.removeChild(_this.mVideo);
            callback(false);
        });
        this.mVideo.src = this.VideoMessageConfig.url;
        this.mVideo.style.position = 'fixed';
        this.mVideo.style.top = '-4000px';
        this.mVideo.style.left = '-4000px';
        this.mVideo.preload = 'auto';
        this.mVideo.loop = true;
        document.body.append(this.mVideo);
    };
    VideoMessage.prototype.onMeasure = function (ctx, trackInfo) {
        if (!this.mIsPlay) {
            this.mIsPlay = true;
            this.mVideo.play();
        }
        return {
            width: this.VideoMessageConfig.width,
            height: this.VideoMessageConfig.height
        };
    };
    VideoMessage.prototype.onLayout = function (ctx, trackInfo) {
        console.log(this.size);
        if (!this.isLayout) {
            this.isLayout = true;
            return {
                top: trackInfo.top,
                left: ctx.canvas.width
            };
        }
        this.position.left -= 3;
        return this.position;
    };
    VideoMessage.prototype.onDraw = function (ctx, trackInfo) {
        ctx.drawImage(this.mVideo, this.position.left, this.position.top, this.size.width, this.size.height);
    };
    VideoMessage.prototype.onDestroyed = function () {
        var result = this.position.left < -this.size.width;
        if (result) {
            document.body.removeChild(this.mVideo);
        }
        return result;
    };
    return VideoMessage;
}(BaseMessage));

// Import here Polyfills if needed. Recommended core-js (npm i -D core-js)
// import "core-js/fn/array.find"
var danmakuPlus = {
    Danmaku: Danmaku,
    DanmakuPool: DanmakuPool,
    TextMessage: TextMessage,
    ImgMessage: ImgMessage,
    VideoMessage: VideoMessage,
    DanmaTrack: DanmaTrack
};
//# sourceMappingURL=danmaku-plus.js.map

export default danmakuPlus;
//# sourceMappingURL=danmaku-plus.es5.js.map
